---
date: 02.09.2024
type: math
---
![Lecture](https://www.youtube.com/watch?v=564pn3Caoy)

## Definitions and Basic Concepts

### What is a Differential Equation?

A **differential equation** is an equation that involves an unknown function and its derivatives. It describes how a quantity changes with respect to another (e.g., time, space). Differential equations are widely used in physics, engineering, economics, biology, and many other fields to model various phenomena.

In mathematical terms, a differential equation can be written in the form:

$$
F\left(x, y, \frac{dy}{dx}, \frac{d^2y}{dx^2}, \ldots\right) = 0,
$$

where $y = y(x)$ is the unknown function, and $\frac{dy}{dx}, \frac{d^2y}{dx^2}, \ldots$ are its derivatives.

### What is an ODE?

An **Ordinary Differential Equation (ODE)** is a type of differential equation that involves functions of only one independent variable and its derivatives. The general form of an ODE is:

$$
F\left(x, y, y', y'', \ldots, y^{(n)}\right) = 0,
$$

where $x$ is the independent variable, $y = y(x)$ is the dependent variable, and $y', y'', \ldots, y^{(n)}$ represent the first, second, and $n$-th derivatives of $y$ with respect to $x$.

**Example:**
A simple example of an ODE is the first-order linear ODE:

$$
\frac{dy}{dx} + p(x)y = q(x),
$$

where $p(x)$ and $q(x)$ are given functions.

### What is a Linear and Homogeneous ODE?

- A **linear ODE** is an ODE in which the dependent variable $y$ and its derivatives appear to the first power and are not multiplied together. A general $n$-th order linear ODE can be written as:

  $$
  a_n(x)y^{(n)} + a_{n-1}(x)y^{(n-1)} + \cdots + a_1(x)y' + a_0(x)y = g(x),
  $$

  where $a_i(x)$ are functions of $x$ and $g(x)$ is a given function.

- A **homogeneous ODE** is a special type of linear ODE where $g(x) = 0$. The general form is:

  $$
  a_n(x)y^{(n)} + a_{n-1}(x)y^{(n-1)} + \cdots + a_1(x)y' + a_0(x)y = 0.
  $$

**Example:**
The second-order homogeneous linear ODE:

$$
y'' - 3y' + 2y = 0
$$

is homogeneous because the right-hand side is zero. It can be solved by finding the characteristic equation and determining the general solution.

### What is a Particular Solution of ODEs?

A **particular solution** of an ODE is a specific solution that satisfies both the differential equation and any given initial or boundary conditions. It is different from the **general solution**, which contains arbitrary constants that represent the family of all possible solutions to the differential equation.

To find a particular solution, you substitute the initial or boundary conditions into the general solution and solve for the arbitrary constants.

**Example:**
Consider the ODE:

$$
y'' - 3y' + 2y = 0.
$$

The general solution is:

$$
y(x) = C_1 e^{2x} + C_2 e^x,
$$

where $C_1$ and $C_2$ are arbitrary constants. If we are given initial conditions $y(0) = 1$ and $y'(0) = 0$, we can substitute these into the general solution to find the values of $C_1$ and $C_2$, giving us a **particular solution**.

**Steps to find a Particular Solution:**
1. Find the general solution of the ODE.
2. Use the given initial or boundary conditions to determine the values of the arbitrary constants in the general solution.
3. Substitute these values back into the general solution to get the particular solution.