---
date: 04.09.2024
type: math
---

![Geometric Meaning of differential equations](https://www.youtube.com/watch?v=ccDMpj2UK_M)
## Integral Curve
- **What is it?**

An **integral curve** of a vector field is a curve that is tangent to the vector field at every point. In simpler terms, given a vector field (which can be thought of as arrows pointing in various directions), an integral curve is a path that follows the directions of these arrows.

![Integral Curve in a Vector Field](Integral%20Curve%20in%20a%20Vector%20field.png)

For a vector field $\mathbf{F}(x, y) = (P(x, y), Q(x, y))$ in 2D, an integral curve $\mathbf{r}(t) = (x(t), y(t))$ is a solution to the system of ordinary differential equations:

$$
\frac{dx}{dt} = P(x(t), y(t)), \quad \frac{dy}{dt} = Q(x(t), y(t)).
$$

**Example:**
Consider a simple vector field defined by $\mathbf{F}(x, y) = (y, -x)$. The integral curves of this field are solutions to the differential equations:

$$
\frac{dx}{dt} = y, \quad \frac{dy}{dt} = -x.
$$

Solving this system, we get solutions of the form:

$$
x(t) = A \cos(t) + B \sin(t), \quad y(t) = -A \sin(t) + B \cos(t),
$$

which represent circles centered at the origin.

## Cauchy Problem

![Cauchy formula explanation](https://www.youtube.com/watch?v=phbO46YJ1UQ&t=36s)
The Cauchy problem is a fundamental concept in the study of partial differential equations (PDEs[^1]}). It refers to the problem of finding a solution to a PDE given initial conditions along a certain hypersurface[^2].

- **How do we solve it?**

To solve a Cauchy problem for a PDE, we generally follow these steps:

1. **Formulate the PDE and Initial Conditions:**
   Define the PDE and the initial conditions. The initial conditions are given on a hypersurface, such as a line (in 2D) or a plane (in 3D). For example, consider the wave equation in one dimension:

   $$
   \frac{\partial^2 u}{\partial t^2} - c^2 \frac{\partial^2 u}{\partial x^2} = 0.
   $$

   The initial conditions could be:

   $$
   u(x, 0) = f(x), \quad \frac{\partial u}{\partial t}(x, 0) = g(x),
   $$

   where$f(x)$and$g(x)$are given functions.

2. **Find a General Solution:**
   Solve the PDE using a method that applies to the type of PDE (e.g., separation of variables, Fourier transforms, or characteristic methods). For the wave equation, the general solution can be written using d'Alembert's formula:

   $$
   u(x, t) = \frac{1}{2} \left( f(x - ct) + f(x + ct) \right) + \frac{1}{2c} \int_{x - ct}^{x + ct} g(s) \, ds.
   $$

3. **Apply the Initial Conditions:**
   Substitute the initial conditions into the general solution to find specific forms of the arbitrary functions or constants.

4. **Verify the Solution:**
   Check that the obtained solution satisfies both the PDE and the initial conditions.

**Example:**
For the heat equation in one dimension:

$$
\frac{\partial u}{\partial t} = \alpha \frac{\partial^2 u}{\partial x^2},
$$

with initial condition$u(x, 0) = f(x)$, the solution is:

$$
u(x, t) = \frac{1}{\sqrt{4 \pi \alpha t}} \int_{-\infty}^\infty e^{-\frac{(x - \xi)^2}{4 \alpha t}} f(\xi) \, d\xi,
$$

which uses a convolution of the initial condition$f(x)$with a Gaussian kernel.


[^1]: Partial differential equation
[^2]: A **hypersurface** is a generalization of the concept of a surface to higher dimensions. Similar to matrices in linear algebra. In 3D, it's a plane, in 2D it's a curve/line.