---
date: 09.09.2024
type: math
---

## Method of Variation of Constants
![Variation of parameters](https://www.youtube.com/watch?v=Ik3YW1JGr_A&pp=ygUgTWV0aG9kIG9mIFZhcmlhdGlvbiBvZiBDb25zdGFudHM%3D)
The **method of variation of constants** is a technique used to find a particular solution to a non-homogeneous linear differential equation. This method generalizes the solution of homogeneous equations by allowing the constants in the general solution to vary as functions of the independent variable. 

Notice how it's similar to [Recurrence relations](Discrete%20Structures/Recurrence%20relations.md) 


1. **Solve the homogeneous equation:** Start by solving the associated homogeneous differential equation. For an ODE of the form:

   $$
   y'' + p(x)y' + q(x)y = g(x),
   $$

   solve the homogeneous part:

   $$
   y'' + p(x)y' + q(x)y = 0.
   $$

   The general solution to the homogeneous equation will be:

   $$
   y_h(x) = C_1 y_1(x) + C_2 y_2(x),
   $$

   where $y_1(x)$ and $y_2(x)$ are linearly independent solutions.

2. **Replace constants with functions:** Replace the constants $C_1$ and $C_2$ with functions $u_1(x)$ and $u_2(x)$:

   $$
   y_p(x) = u_1(x)y_1(x) + u_2(x)y_2(x).
   $$

3. **Set up equations for $u_1(x)$ and $u_2(x)$:** Differentiate $y_p(x)$ and use the condition that $u_1'(x)y_1(x) + u_2'(x)y_2(x) = 0$ to avoid second derivatives of $u_1(x)$ and $u_2(x)$. This gives:

   $$
   u_1'(x)y_1(x) + u_2'(x)y_2(x) = 0,
   $$
   $$
   u_1'(x)y_1'(x) + u_2'(x)y_2'(x) = g(x).
   $$

4. **Solve for $u_1'(x)$ and $u_2'(x)$:** Solve this system of equations to find $u_1'(x)$ and $u_2'(x)$.

5. **Integrate to find $u_1(x)$ and $u_2(x)$:** Integrate to find $u_1(x)$ and $u_2(x)$.

6. **Form the particular solution:** The particular solution is:

   $$
   y_p(x) = u_1(x)y_1(x) + u_2(x)y_2(x).
   $$

## Bernoulli Equation
![Understanding the bernoulli equation](https://www.youtube.com/watch?v=DW4rItB20h4)
![Using the bernoulli equation](https://www.youtube.com/watch?v=iCN8nGXE29o)

A **Bernoulli equation** is a type of first-order nonlinear differential equation of the form:

$$
\frac{dy}{dx} + P(x)y = Q(x)y^n,
$$

where $n \neq 0, 1$.

- **When and how do we apply it?**

To solve a Bernoulli equation:

1. **Divide through by $y^n$:**

   $$
   y^{-n} \frac{dy}{dx} + P(x)y^{1-n} = Q(x).
   $$

2. **Make a substitution:** Let $v = y^{1-n}$. Then $\frac{dv}{dx} = (1-n)y^{-n} \frac{dy}{dx}$.

3. **Rewrite the equation in terms of $v$:**

   $$
   \frac{dv}{dx} + (1-n)P(x)v = (1-n)Q(x).
   $$

   This is now a linear differential equation in $v(x)$.

4. **Solve the linear ODE for $v$:**

   Use an integrating factor to solve for $v(x)$.

5. **Substitute back to find $y(x)$:**

   Since $v = y^{1-n}$, solve for $y(x)$.

## Riccati Equation
![Explanation](https://www.youtube.com/watch?v=MoO7Jw06_PM)

A **Riccati equation** is a first-order nonlinear differential equation of the form:

$$
\frac{dy}{dx} = a(x) + b(x)y + c(x)y^2.
$$

- **When do we use it?**

Riccati equations are used in various fields such as control theory and fluid dynamics. They can sometimes be solved by making an appropriate substitution if a particular solution is known. In general, Riccati equations do not have a straightforward general solution like linear ODEs.

## $n \geq 2$ Linear ODE

![Constant coeff DEs](https://www.youtube.com/watch?v=is0F0u62IbY)
- **What do we do with those?**

For linear ODEs of order $n \geq 2$, we typically look for a general solution that is a linear combination of $n$ linearly independent solutions.

### General Properties of Spaces of Solutions of such $\epsilon$

- **Linear dependence:** Solutions $y_1(x), y_2(x), \ldots, y_n(x)$ are linearly independent if no solution can be written as a linear combination of the others.

- **Dimension:** The solution space of a linear homogeneous ODE of order $n$ has dimension $n$.

- **Fundamental theorem:** If $y_1(x), y_2(x), \ldots, y_n(x)$ are $n$ linearly independent solutions to an $n$-th order linear homogeneous ODE, then any solution can be written as:

  $$
  y(x) = C_1 y_1(x) + C_2 y_2(x) + \cdots + C_n y_n(x),
  $$

  where $C_1, C_2, \ldots, C_n$ are constants.

- **Structure of the space:** The space of solutions is a vector space, where each solution can be represented as a linear combination of a set of basis solutions.
