---
type: theoretical
---
## Proving Equivalences

To prove that two statements $P$ and $Q$ are equivalent ($P \iff Q$):

1. ***Chain of Equivalences***:
   Show that $P(x_1, \ldots, x_n) \iff \ldots \iff Q(x_1, \ldots, x_n)$, transforming $P$ into $Q$ step by step.

2. ***Bi-conditional***[^10] ***Decomposition***[^1]:
   Use the fact that:
   $$
   (P \iff Q) \iff ((P \implies Q) \land (Q \implies P)).
   $$
   - Prove $P \implies Q$ (first direction).
   - Prove $Q \implies P$ (second direction).

---

## Proving Universal Statements

To prove a statement of the form $\forall x\, P(x)$:

1. Proof by Exhaustion[^5]:
	- If the domain of $x$ is finite, verify $P(x)$ for each possible value.

2. Proof by Universal Generalization[^6]:
	- Let $c$ be an arbitrary element of the domain.
	- Prove $P(c)$.
	- Conclude that $\forall x\, P(x)$ holds.

**Example**:
- For all integers $n$, if $n$ is even, then $n^2$ is even.
- ***Proof***:
	1. Let $n$ be an arbitrary integer.
	2. Assume $n$ is even. Then $n = 2k$ for some integer $k$.
	3. Compute $n^2 = (2k)^2 = 4k^2 = 2(2k^2)$, which is even.

---

## Proving Existential Statements

To prove a statement of the form $\exists x\, P(x)$:

1. Constructive Proof[^7]:
   Find a specific $c$ such that $P(c)$ is true.

2. Non-constructive Proof[^8]:
	- Assume $\forall x\, \neg P(x)$ (negation of existence).
	- Derive a contradiction.
	- Conclude that $\exists x\, P(x)$ must be true.

---

## Proof by Contraposition

Instead of proving $P \implies Q$, prove its contrapositive[^3]:
$$
\neg Q \implies \neg P
$$
**Example**:
- For all $n \in \mathbb{N}$, if $n^2$ is odd, then $n$ is odd.
- ***Proof***:
  1. Prove the contrapositive: If $n$ is even, then $n^2$ is even.
  2. Assume $n = 2k$. Then $n^2 = (2k)^2 = 4k^2 = 2(2k^2)$, which is even.

---

## Proof by Contradiction

To prove $P$:
1. Assume $\neg P$.
2. Derive a contradiction[^4].
3. Conclude that $P$ must be true.

**Example**:
- $\sqrt{2}$ is irrational.
- **Proof**:
  1. Assume $\sqrt{2}$ is rational, so $\sqrt{2} = \frac{p}{q}$, where $p, q \in \mathbb{Z}$, $q \neq 0$, and $\frac{p}{q}$ is in lowest terms.
  2. Square both sides: $2 = \frac{p^2}{q^2}$, so $2q^2 = p^2$.
  3. $p^2$ is even, so $p$ is even ($p = 2k$).
  4. Substitute $p = 2k$: $2q^2 = (2k)^2 = 4k^2$, so $q^2 = 2k^2$.
  5. $q^2$ is even, so $q$ is even.
  6. Contradiction: $p$ and $q$ cannot both be even since $\frac{p}{q}$ is in lowest terms.
  7. Conclude $\sqrt{2}$ is irrational.

---

## Mathematical Induction
[^2]
Generalized in [Proofs (FP)](Proofs.md)


![induction](induction.png)

To prove a statement of the form $\forall n \geq n_0, P(n)$:

1. *Base Case*:
   Prove $P(n_0)$ is true.

2. *Inductive Hypothesis* - $IH$:
   Assume $P(k)$ is true for some $k \geq n_0$.

3. *Inductive Step*:
   Prove $P(k + 1)$ is true using $P(k)$.

**Example**:
- $1 + 2 + \dots + n = \frac{n(n + 1)}{2}$ for all $n \geq 1$.
- ***Proof***:
  1. Base Case: For $n = 1$, LHS = $1$, RHS = $\frac{1(1 + 1)}{2} = 1$. Holds true.
  2. $IH$: Assume $1 + 2 + \dots + k = \frac{k(k + 1)}{2}$.
  3. Inductive Step: Show $1 + 2 + \dots + (k + 1) = \frac{(k + 1)(k + 2)}{2}$.
     $$
     \text{LHS} = (1 + 2 + \dots + k) + (k + 1) = \frac{k(k + 1)}{2} + (k + 1).
     $$
     Simplify:
     $$
     \frac{k(k + 1)}{2} + (k + 1) = \frac{k(k + 1) + 2(k + 1)}{2} = \frac{(k + 1)(k + 2)}{2}.
     $$
     Matches RHS.

---

## Strong[^9] Mathematical Induction

To prove $\forall n \geq n_0, P(n)$:

1. *Base Cases*:
   Prove $P(n_0), P(n_0 + 1), \ldots, P(n_0 + m)$ are true.

2. *Inductive Hypothesis*:
   Assume $P(i)$ is true for all $n_0 \leq i \leq k$.

3. *Inductive Step*:
   Prove $P(k + 1)$ is true using the assumption $P(i)$ for all $i \leq k$.

**Example**:
- Every $n \geq 2$ can be factored into primes.
- **Proof**:
  1. *Base Case*: $n = 2$ is a prime.
  2. $IH$: Assume every $n \leq k$ can be factored into primes.
  3. *Inductive Step*: For $n = k + 1$:
     - If $k + 1$ is prime, done.
     - If composite, $k + 1 = a \cdot b$, where $2 \leq a, b \leq k$.
     - By hypothesis, $a$ and $b$ can be factored into primes.
     - Combine the prime factors of $a$ and $b$ to get $k + 1$'s factorization.




[^1]: Breaking a complex problem into smaller, simpler parts to solve each one step by step
[^2]: Proof that works by showing it works for the smallest case, then assuming it works for one number and proving it works for the next
[^3]: Instead of directly proving "If A, then B," you prove "If not B, then not A," which means the same thing logically.
[^4]: A way of proving something is true by assuming it is false and showing this leads to a logical impossibility, which, as we know, really messes with everything.
[^5]: Checking every possible case individually to prove something is true for all of them.
[^6]: Showing something is true for all cases by proving it for an arbitrary or random example. Similar to formal proof for introducing the $\forall$ quantifier
[^7]: Directly finding an example to show something exists or is true.
[^8]: Proving something exists or is true without giving a specific example, usually by ruling out the possibility of it not being true.
[^9]: Like induction, but you assume everything is true up to a certain point to prove the next case
[^10]: A statement where both directions are true, like "A if and only if B," meaning A leads to B, and B leads to A.
